package org.expeditee.agents.mail;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Date;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.Flags.Flag;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.expeditee.gui.AttributeValuePair;
import org.expeditee.gui.Frame;
import org.expeditee.gui.MessageBay;
import org.expeditee.items.Text;

public class MailSession {
	public static boolean _autoConnect = true;

	private static MailSession _theMailSession = null;

	private Session _session;

	private Transport _transport;

	private Store _store;

	private Folder _folder;

	private String _address;

	private String _username;

	private String _password;

	private String _popServer;

	private Boolean _bConnecting;

	private MailSession(Frame settingsFrame) {
		_bConnecting = false;

		Properties props = System.getProperties();

		_username = null;
		_password = "";
		_popServer = null;

		// Set the settings
		for (Text item : settingsFrame.getBodyTextItems(false)) {
			AttributeValuePair avp = new AttributeValuePair(item.getText());
			if (!avp.hasPair())
				continue;
			String attributeFullCase = avp.getAttribute();
			String attribute = attributeFullCase.toLowerCase();

			if (attribute.equals("user")) {
				_username = avp.getValue();
				props.setProperty("mail.user", _username);
			} else if (attribute.equals("password")) {
				_password = avp.getValue();
				props.setProperty("mail.password", _password);
			} else if (attribute.equals("address")) {
				_address = avp.getValue();
			} else if (attribute.equals("smtpserver")) {
				props.setProperty("mail.transport.protocol", "smtp");
				props.setProperty("mail.host", avp.getValue());
				props.setProperty("mail.smtp.starttls.enable", "true");
				props.setProperty("mail.smtp.host", avp.getValue());
				props.setProperty("mail.smtp.auth", "true");
			} else if (attribute.equals("popserver")) {
				_popServer = avp.getValue();
			}
		}

		// Create the authenticator
		Authenticator auth = null;
		if (_username != null) {
			auth = new SMTPAuthenticator(_username, _password);
		}
		java.security.Security
				.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

		// -- Attaching to default Session, or we could start a new one --
		_session = Session.getDefaultInstance(props, auth);
		try {
			// Set up the mail receiver
			_store = _session.getStore("pop3s");

			// Connect the mail sender
			_transport = _session.getTransport();
			if (_autoConnect) {
				connectThreaded();
			}
		} catch (Exception e) {
			MessageBay.errorMessage("Error in ExpMail setup");
		}
	}

	/**
	 * Attempts to connect the mail transporter to the mail server.
	 * 
	 */
	public static void connect() {
		if (_theMailSession._bConnecting) {
			MessageBay.errorMessage("Already connecting to mail server");
			return;
		} else if (_theMailSession != null) {
			_theMailSession.connectThreaded();
		}
	}

	private void connectThreaded() {
		Thread t = new ConnectThread(this);
		t.start();
	}

	public synchronized void connectServers() {
		try {
			if (!_transport.isConnected()) {
				MessageBay.displayMessage("Connecting to SMTP server...");
				_bConnecting = true;
				_transport.connect();
				MessageBay.overwriteMessage("SMTP server connected");
			} else {
				MessageBay.warningMessage("SMTP server already connected");
			}
		} catch (MessagingException e) {
			MessageBay.errorMessage("Error connecting to SMTP server");
		}

		if (_popServer != null && !_store.isConnected()) {
			try {
				MessageBay.displayMessage("Connecting to POP3 server...");
				_store.connect(_popServer, _username, _password);

				// -- Try to get hold of the default folder --
				_folder = _store.getDefaultFolder();
				if (_folder == null)
					throw new Exception("No default folder");
				// -- ...and its INBOX --
				_folder = _folder.getFolder("INBOX");
				if (_folder == null)
					throw new Exception("No POP3 INBOX");
				// -- Open the folder for read only --
				_folder.open(Folder.READ_ONLY);
				MessageBay.overwriteMessage("POP3 server connected");
				MessageBay.warningMessage(getUnreadMessageCount() + " unread messages are in your inbox");
			} catch (Exception e) {
				// e.printStackTrace();
				MessageBay.errorMessage("Error connecting to POP3 server");
			}
		}
		_bConnecting = false;
	}

	public static boolean sendTextMessage(String to, String cc, String bcc,
			String subject, String body) {

		if (_theMailSession == null) {
			MessageBay.errorMessage("Add mail settings to profile frame");
			return false;
		}

		if (_theMailSession._bConnecting) {
			MessageBay.errorMessage("Busy connecting to mail server...");
			return false;
		}

		return _theMailSession.sendText(to, cc, bcc, subject, body);
	}

	private synchronized boolean sendText(String to, String cc, String bcc,
			String subject, String body) {
		if (!_transport.isConnected()) {
			MessageBay
					.warningMessage("Not connected to server, attempting to reconnect...");
			try {
				_bConnecting = true;
				_transport.connect();
				_bConnecting = false;
			} catch (Exception e) {
				MessageBay.errorMessage("Could not connect to mail server");
				_bConnecting = false;
				return false;
			}
		}

		if (to == null) {
			MessageBay.errorMessage("Add tag @to:<sendToEmailAddress>");
			return false;
		}

		try {
			// -- Create a new message --
			Message msg = new MimeMessage(_session);

			// -- Set the FROM and TO fields --
			msg.setFrom(new InternetAddress(_address));
			msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(
					to, false));

			// -- We could include CC recipients too --
			if (cc != null) {
				msg.setRecipients(Message.RecipientType.CC, InternetAddress
						.parse(cc, false));
			}

			if (bcc != null) {
				msg.setRecipients(Message.RecipientType.BCC, InternetAddress
						.parse(bcc, false));
			}

			// -- Set the subject and body text --
			msg.setSubject(subject);
			msg.setContent(body.toString(), "text/plain");

			// -- Set some other header information --
			msg.setHeader("ExpMail", "Expeditee");
			msg.setSentDate(new Date());

			Transport.send(msg, msg.getRecipients(Message.RecipientType.TO));
		} catch (Exception e) {
			MessageBay.errorMessage("Error sending mail: " + e.getMessage());
			return false;
		}
		MessageBay.displayMessage("Message sent OK.");
		return true;
	}

	public static void init(Frame settingsFrame) {

		if (settingsFrame == null)
			return;

		if (_theMailSession == null)
			_theMailSession = new MailSession(settingsFrame);
	}

	private class SMTPAuthenticator extends javax.mail.Authenticator {
		private String _username;

		private String _password;

		public SMTPAuthenticator(String username, String password) {
			_username = username;
			_password = password;
		}

		@Override
		public PasswordAuthentication getPasswordAuthentication() {
			return new PasswordAuthentication(_username, _password);
		}
	}

	public static MailSession getInstance() {
		return _theMailSession;
	}

	public synchronized void finalise() {
		try {
			if (_transport != null && _transport.isConnected()) {
				_transport.close();
			}

			if (_folder != null) {
				_folder.close(false);
			}

			if (_store != null) {
				_store.close();
			}
		} catch (Exception e) {

		}
	}

	public int getUnreadMessageCount() {
		// -- Get the message wrappers and process them --
		Message[] msgs;
		int count = 0;
		try {
			msgs = _folder.getMessages();
			for (int msgNum = 0; msgNum < msgs.length; msgNum++) {
				if (msgs[msgNum].getFlags().contains(Flag.SEEN))
					continue;
				count++;
			}
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return count;
	}

	public String getMail() {
		StringBuffer sb = new StringBuffer();
		// -- Get the message wrappers and process them --
		Message[] msgs;
		try {
			msgs = _folder.getMessages();
			for (int msgNum = 0; msgNum < msgs.length; msgNum++) {
				if (msgs[msgNum].getFlags().contains(Flag.SEEN))
					continue;
				// Only get messages that have not been read
				sb.append(getTextMessage(msgs[msgNum]));
				sb.append('\n');
			}
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		sb.deleteCharAt(sb.length() - 1);
		return sb.toString();
	}

	/**
	 * "getTextMessage()" method to print a message.
	 */
	public String getTextMessage(Message message) {
		StringBuffer sb = new StringBuffer();
		
		try {
			// Get the header information
			String from = ((InternetAddress) message.getFrom()[0])
					.getPersonal();
			if (from == null)
				from = ((InternetAddress) message.getFrom()[0]).getAddress();
			sb.append("FROM: " + from);
			String subject = message.getSubject();
			sb.append("SUBJECT: " + subject);
			// -- Get the message part (i.e. the message itself) --
			Part messagePart = message;
			Object content = messagePart.getContent();
			// -- or its first body part if it is a multipart message --
			if (content instanceof Multipart) {
				messagePart = ((Multipart) content).getBodyPart(0);
				System.out.println("[ Multipart Message ]");
			}
			// -- Get the content type --
			String contentType = messagePart.getContentType();
			// -- If the content is plain text, we can print it --
			//System.out.println("CONTENT:" + contentType);
			if (contentType.startsWith("text/plain")
					|| contentType.startsWith("text/html")) {
				InputStream is = messagePart.getInputStream();
				BufferedReader reader = new BufferedReader(
						new InputStreamReader(is));
				String thisLine = reader.readLine();
				while (thisLine != null) {
					sb.append(thisLine);
					thisLine = reader.readLine();
				}
			}
			sb.append("-----------------------------");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return sb.toString();
	}
}
